package cn.efunbox.controller;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.json.MappingJacksonJsonView;

import cn.efunbox.model.Film;
import cn.efunbox.model.Recommend;
import cn.efunbox.service.IFilmService;
import cn.efunbox.service.IRecommendService;

@SuppressWarnings("deprecation")
@Controller
@RequestMapping("/film")
public class FilmController {
	 
	@Autowired  
	private  HttpServletRequest request;
	
    private IFilmService filmService;
    @Autowired
    private IRecommendService recmdService;
    
    public IFilmService getFilmService() {
        return filmService;
    }

    @Autowired
    public void setFilmService(IFilmService filmService) {
        this.filmService = filmService;
    }

    @RequestMapping("/getAll")
    @ResponseBody
    public Map<String,Object> getAll(){
        List<Film> rows = filmService.loadAll();
        Map<String, Object> map = new HashMap<String, Object>(2);
        map.put("code", 200);
        map.put("rows", rows);
        return map;
    }

    @RequestMapping("/delete")
    @ResponseBody
    public Map<String,Object> delete(){
    	Map<String, Object> map = new HashMap<String, Object>(2);
        String film_code = request.getParameter("film_code")==null?"":request.getParameter("film_code").trim();
        String type = request.getParameter("type")==null?"1":request.getParameter("type").trim();
		if(film_code.isEmpty() || type.isEmpty()){
			map.put("code", 301);
			map.put("msg", "film_code或type参数错误");
			return map;
		}
    	filmService.delete(film_code, Integer.valueOf(type));
    	recmdService.delete(Integer.valueOf(type), film_code, 0, 0);
        map.put("code", 200);
        return map;
    }

    @RequestMapping("/add")
    @ResponseBody
    public Map<String,Object> add(){
    	Map<String, Object> map = new HashMap<String, Object>(2);
        String type = request.getParameter("type")==null?"1":request.getParameter("type").trim();
        String film_code = request.getParameter("film_code")==null?"":request.getParameter("film_code").trim();
		String film_type = request.getParameter("film_type")==null?"1":request.getParameter("film_type").trim();
		String mark_type = request.getParameter("mark_type")==null?"0":request.getParameter("mark_type").trim();
		String is_vip = request.getParameter("is_vip")==null?"0":request.getParameter("is_vip").trim();
		String film_grade = request.getParameter("film_grade")==null?"5":request.getParameter("film_grade").trim();
		String film_star = request.getParameter("film_star")==null?"5":request.getParameter("film_star").trim();
		String age_group = request.getParameter("age_group")==null?"0":request.getParameter("age_group").trim();
		String series_count = request.getParameter("series_count")==null?"1":request.getParameter("series_count").trim();
		String publish_date = request.getParameter("publish_date")==null?"0":request.getParameter("publish_date").trim();
		String online_date = request.getParameter("online_date")==null?"0":request.getParameter("online_date").trim();
		String area = request.getParameter("area")==null?"0":request.getParameter("area").trim();
		String movie_type = request.getParameter("movie_type")==null?"0":request.getParameter("movie_type").trim();
		String target_people = request.getParameter("target_people")==null?"0":request.getParameter("target_people").trim();
		String language = request.getParameter("language")==null?"0":request.getParameter("language").trim();
		String duration = request.getParameter("duration")==null?"0":request.getParameter("duration").trim();
		if(film_code.isEmpty() || type.isEmpty()){
			map.put("code", 301);
			map.put("msg", "film_code或type参数错误");
			return map;
		}
		int curTime = (int)(System.currentTimeMillis()/1000);
		
		Film old = filmService.get(film_code, Integer.valueOf(type));
		if(old!=null){
			map.put("code", 302);
			map.put("msg", "节目已存在，不能重复添加");
			return map;
		}
		Film film = new Film();
		film.setType(Integer.valueOf(type));
		film.setAge_group(Integer.valueOf(age_group));
		film.setArea(Integer.valueOf(area));
		film.setCreated(curTime);
		film.setDuration(Integer.valueOf(duration));
		film.setFilm_code(film_code);
		film.setFilm_grade(Float.valueOf(film_grade));
		film.setFilm_star(Float.valueOf(film_star));
		film.setFilm_type(Integer.valueOf(film_type));
		film.setIs_vip(Integer.valueOf(is_vip));
		film.setLanguage(Integer.valueOf(language));
		film.setMark_type(Integer.valueOf(mark_type));
		film.setMovie_type(Integer.valueOf(movie_type));
		film.setOnline_date(Integer.valueOf(online_date));
		film.setPublish_date(Integer.valueOf(publish_date));
		film.setSeries_count(Integer.valueOf(series_count));
		film.setTarget_people(Integer.valueOf(target_people));
		
		int ret = filmService.insert(film);
		
		for(int age=0;age<20;age++){
			int score = initScore(film, age);
			Recommend recmd = new Recommend();
			recmd.setAge(age);
			recmd.setCreated(curTime);
			recmd.setFilm_code(film_code);
			recmd.setScore(score);
			recmd.setSex(1);
			recmd.setType(Integer.valueOf(type));
			recmdService.insert(recmd);
			
			recmd.setSex(0);
			recmdService.insert(recmd);
		}
		
		map.put("code", 200);
        map.put("rows", ret);
        return map;
    }
    
    private int initScore(Film film, int age){
    	int curTime = (int)(System.currentTimeMillis()/1000);
    	int score = 0;
    	score += film.getMark_type()==1?5:0;				//最多5分
		score += film.getFilm_type()==1?5:0;				//5
		score += film.getIs_vip()==1?5:0;					//5
		score += 3 * Math.ceil(Float.valueOf(film.getFilm_grade()));//15
		score += 2 * Math.ceil(Float.valueOf(film.getFilm_star()));	//10
		int ageDis =  age - film.getAge_group();
		if(ageDis==0)
			ageDis = 1;
		score += Math.ceil(30/ageDis);						//30
		score += Math.max(0, 15-2*Math.max(0, (curTime-film.getPublish_date())/(3600*24*365)));//15
		score += Math.max(0, 15-2*Math.max(0, (curTime-film.getOnline_date())/(3600*24*365)));//15
		if(score<0)
			score = 0;
		else if(score>100)
			score = 100;
		
    	return score;
    }
}